@@ -12,6 +12,7 @@
 //---------------------------------------------------------------------------
 #include "MediaInfo/File__Analyze.h"
 #include "MediaInfo/File__Duplicate.h"
+#include "MediaInfo/TimeCode.h"
 #include <cmath>
 //---------------------------------------------------------------------------
 
@@ -32,7 +33,6 @@ class File_Avc :
 public :
     //In
     int64u Frame_Count_Valid;
-    bool   FrameIsAlwaysComplete;
     bool   MustParse_SPS_PPS;
     bool   SizedBlocks;
 
@@ -513,6 +513,9 @@ private :
         bool                            GA94_03_IsPresent;
     #endif //defined(MEDIAINFO_DTVCCTRANSPORT_YES)
 
+    //Misc
+    TimeCode                            TC_Current;
+
     //Replacement of File__Analyze buffer
     const int8u*                        Buffer_ToSave;
     size_t                              Buffer_Size_ToSave;
@@ -541,6 +544,8 @@ private :
     size_t                              Interlaced_Bottom;
     size_t                              Structure_Field;
     size_t                              Structure_Frame;
+    size_t                              Slices_CountInThisFrame;
+    size_t                              MaxSlicesCount;
 
     //Temp
     Ztring                              Encoded_Library;
